package org.tbasket.handler

import org.tbasket.auth.Authentificator
import org.tbasket.data.User
import org.tbasket.error.RegularException.InvalidRequest
import org.tbasket.handler.HandlerUtils.parseAttribute
import zio.ZIO
import zio.http.model.{Cookie, Headers, Status}
import zio.http.{Request, Response, model}
import zio.json.ast.{Json, JsonCursor}
import zio.json.*

object RegisterPageHandler extends PageHandler {
  
  private def tryPost(request: Request) =
    for
      body <- request.body.asString
        .mapError(e => InvalidRequest("Invalid request body", e.getMessage))
    
      json <- ZIO.fromEither(body.fromJson[Json])
        .mapError(InvalidRequest("Invalid JSON body", _))
    
      name <- parseAttribute(json, "name", JsonCursor.field("name").isString)
      forename <- parseAttribute(json, "forename", JsonCursor.field("forename").isString)
      mail <- parseAttribute(json, "email", JsonCursor.field("email").isString)
      password <- parseAttribute(json, "password", JsonCursor.field("password").isString)
    
      user <- ZIO.serviceWithZIO[Authentificator](_.registerUser(name, forename, mail, password))
      jwt <- ZIO.serviceWithZIO[Authentificator](_.requestNewJwt(user))
    yield Response(
      status = Status.Found,
      headers = Headers.location("/") ++ //register successful, go back to main page
        Headers.setCookie(Cookie("JWT", jwt))
    )
  
  def post(request: Request) = tryPost(request).catchSome {
    case x => ???
  }
  
}
